;;; -*- TI-Asm -*-
;;;
;;; DuckSign - sign Flash applications on the TI-83 Plus
;;;
;;; Copyright (C) 2010 Benjamin Moody
;;;
;;; This program is free software: you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License as
;;; published by the Free Software Foundation; either version 3 of the
;;; License, or (at your option) any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;; General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program.  If not, see <http://www.gnu.org/licenses/>.

	processor z80

INTERNAL_VALIDATE	equ 1
INTERNAL_MD5		equ 1

; ENABLE_TIMER		equ 1
; ENABLE_ASSERTIONS	equ 1
; ENABLE_LOGGING		equ 1
; LOG_PORT		equ 0FFh

	nolist
	include <ti83plus.inc>
	include "defs.inc"
	include "debug.inc"
	list

appFValue equ 83E6h

rabinF_length	equ appFValue
rabinF		equ rabinF_length + 1

	org appData

;;; Temporary storage for math routines

sqrtab:			ds 512	; Squaring table
  assert low(sqrtab) == 0

modulusShift0:		ds 36	; Shifted modulus
  assert high($) == high(modulusShift0)

;;; Counters used for timing

interruptCount:		ds 4	; Number of interrupts

timerExpiredCount:	ds 4	; Number of times programmable timer
				; has expired

	assert $ <= ramCodeEnd

	org tempSwapArea

;;; Important permanent storage

rabinSignature_length:	ds 1
rabinSignature:		ds 66	; Final app signature
  assert high($) == high(rabinSignature)

progressXBit:		ds 1	; Current bit for progress bar
progressX:		ds 1	; Current LCD column for progress bar
progressCount:		ds 2	; Time to next progress bar update
progressTotal:		ds 2	; Total number of progress units

appMenuStart:		ds 1	; Starting position in menu
appMenuCurrent:		ds 1	; Cursor position in menu

appStartPage:		ds 1	; Starting page of the app
appNumPages:		ds 1	; Number of pages

appFinalPageLength:	ds 2	; Length of final page

	align 256

;;; More temporary storage for math routines

rabinM:			ds 33	; Hash to be signed
  assert high($) == high(rabinM)

rabinX:			ds 33	; Square root of M mod P
  assert high($) == high(rabinX)

rabinY:			ds 33	; Square root of M mod Q
  assert high($) == high(rabinY)

modulusShift1:		ds 36	; Shifted modulus
  assert high($) == high(modulusShift1)

	assert $ <= tempSwapArea + 232

	org saveSScreen

;;; Still more temporary storage for math routines

mul_temp_64:		ds 16	; Scratch space for Mul64by64
mul_diffA_64 equ mul_temp_64 + 8
mul_diffB_64 equ mul_temp_64 + 12
  assert high($) == high(mul_temp_64)

	align	256

modJumpTable:		ds 256	; Jump table for ReduceModN
  assert low(modJumpTable) == 0

mul_temp_128:		ds 32	; Scratch space for Mul128by128
mul_diffA_128 equ mul_temp_128 + 16
mul_diffB_128 equ mul_temp_128 + 24
  assert high($) == high(mul_temp_128)

mul_temp_256:		ds 64	; Scratch space for Mul256by256
mul_diffA_256 equ mul_temp_256 + 32
mul_diffB_256 equ mul_temp_256 + 48
  assert high($) == high(mul_temp_256)

modulusShift2:		ds 36	; Shifted modulus
  assert high($) == high(modulusShift2)

modulusShift3:		ds 36	; Shifted modulus
  assert high($) == high(modulusShift3)

modulusShift4:		ds 36	; Shifted modulus
  assert high($) == high(modulusShift4)

modulusShift5:		ds 36	; Shifted modulus
  assert high($) == high(modulusShift5)

	align	256

modulusShift6:		ds 36	; Shifted modulus
  assert high($) == high(modulusShift6)

modulusShift7:		ds 36	; Shifted modulus
  assert high($) == high(modulusShift7)

rabinTemp1:		ds 66	; Scratch space for RabinSqrt
  assert high($) == high(rabinTemp1)

rabinTemp2:		ds 66
  assert high($) == high(rabinTemp2)

rabinSqrtOutput:	ds 2	; Pointer to output buffer for
				; RabinSqrt

modulus:		ds 2	; Pointer to actual modulus

modulusLength:		ds 1	; Length of modulus

	assert $ <= saveSScreen + 768

	org	userMem - 2
	db	0BBh, 6Dh

	xor	a
	jr	nc, Start
	;; UNREACHABLE
	db	"DuckSign: sign Flash apps", 0

Start:
	BCALL	_RunIndicOff
	in	a, (6)
	ld	b, a
	in	a, (20h)
	ld	c, a
	push	bc
	 ld	c, 1
	 call	SetCPUSpeed
	 call	Main
	 BCALL	_ClrScrnFull
	 BCALL	_HomeUp
	 pop	bc
  ifdef ENABLE_TIMER
	im	1
  endif
WaitForOnKey:
	ei
	halt
	call	CheckOnAbort
	jr	c, WaitForOnKey
	res	onInterrupt, (iy + onFlags)
	ld	a, b
	out	(6), a
SetCPUSpeed:
	in	a, (2)
	add	a, a
	ret	nc
	ld	a, c
	out	(20h), a
	ret

	include	"addsub.asm"
	include	"mod.asm"
	include	"key.asm"
	include	"rabin.asm"
	include	"mul.asm"
	include	"md5.asm"
	include	"hash.asm"
	include "main.asm"
	include "sign.asm"
	include	"apps.asm"
	include	"progress.asm"
	include	"util.asm"
	include	"timer.asm"
	include "log.asm"
	include	"data.asm"
