;;; -*- TI-Asm -*-
;;;
;;; DuckSign - sign Flash applications on the TI-83 Plus
;;;
;;; Copyright (C) 2010 Benjamin Moody
;;;
;;; This program is free software: you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License as
;;; published by the Free Software Foundation; either version 3 of the
;;; License, or (at your option) any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;; General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program.  If not, see <http://www.gnu.org/licenses/>.

Main:
	BCALL	_FillBasePageTable ; in case the program is run right
				   ; after reset - unlikely but
				   ; possible - or the table was
				   ; corrupted
	call	CheckBattery
	ret	nc
	call	GetFirstApp
	call	SelectApp
	ret	c
	ld	h, a
	ld	l, a
Main_SetMenuPos:
	ld	(appMenuStart), hl
Main_Restart:
	ld	hl, 0
	ld	(curRow), hl
	ld	a, (appMenuStart)
Main_DisplayLoop:
	push	af
	 ld	hl, (appMenuCurrent)
	 cp	l
	 ld	a, ' '
	 jr	nz, Main_NotSelected
	 ld	a, Lconvert
Main_NotSelected:
	 BCALL	_PutC
	 pop	af
	push	af
	 call	GetAppName
	 BCALL	_PutS
	 BCALL	_EraseEOL
	 ld	a, (curRow)
	 cp	7
	 jr	nc, Main_DisplayDone
	 BCALL	_NewLine
	 pop	af
	call	GetNextApp
	jr	nc, Main_DisplayLoop
	push	bc
Main_DisplayDone:
	 BCALL	_EraseEOW
Main_KeyLoop1:
	 pop	bc
Main_KeyLoop:
	call	WaitCSC
	ld	hl, (appMenuStart)
	cp	skUp
	jr	z, Main_Up
	cp	skDown
	jr	z, Main_Down
	cp	sk2nd
	jr	z, Main_Select
	cp	skEnter
	jr	z, Main_Select
	cp	skMode
	ret	z
	cp	skClear
	jr	nz, Main_KeyLoop
	ret

Main_Up:
	ld	a, h
	call	GetPrevApp
	jr	c, Main_KeyLoop
	ld	h, a
	ld	a, l
	cp	h
	jr	nc, Main_SetMenuPos
	ld	l, h
	jr	Main_SetMenuPos

Main_Down:
	ld	a, (appStartPage)
	cp	h
	push	af
	 ld	a, h
	 call	GetNextApp
	 jr	c, Main_KeyLoop1
	 ld	h, a
	 pop	af
	jr	nz, Main_SetMenuPos
	ld	a, l
	call	GetNextApp
	ld	l, a
	jr	Main_SetMenuPos

Main_ShowErrorMessage:
	call	ShowMessageWait
	jr	Main_Restart

Main_Select:
	;; Check that app length is reasonable
	ld	a, h
	call	GetAppSize
	ld	hl, invalidLengthMsg
	jr	nz, Main_ShowErrorMessage

	ld	de, 4000h
	ld	a, (de)
	or	a
	ld	hl, appDeletedMsg
	jr	z, Main_ShowErrorMessage

	;; Check that app key is set to 0104
	in	a, (6)
	ex	de, hl
	ld	de, 8010h
	BCALL	_FindAppHeaderSubField
	ld	de, id0104
	ld	b, 4
	call	z, CompStrsN
	ld	hl, unknownKeyMsg
	jr	nz, Main_ShowErrorMessage

	;; Compute hash of app
	ld	hl, hashingMsg
	call	DispProgressMessage

	TIMER_START
	call	HashCurApp
	TIMER_STOP
	ret	c

	;; Check if app is already signed
	call	ReadAppSignature
	ld	hl, badSignatureMsg
	jr	c, Main_ShowErrorMessage
	jr	z, Main_NoSignature

  ifndef INTERNAL_VALIDATE
	ld	hl, checkingMsg
	call	DispMessage
  endif

	di
	TIMER_START
	call	ValidateSignature
	TIMER_STOP
	ld	hl, alreadySignedMsg
	jr	z, Main_ShowErrorMessage

Main_NoSignature:
	;; Sign application
	ld	hl, signingMsg
	call	DispProgressMessage

	di
	TIMER_START
	call	GenerateSignature
	TIMER_STOP
	ret	c

	call	CheckBattery
	ret	nc

	call	Unlock
	ld	hl, cannotUnlockMsg
	jr	c, Main_ShowFatalMessage
	ld	hl, erasingMsg
	call	DispMessage
	call	EraseAppSignature
	jr	nz, Main_WriteFailed

	ld	hl, writingMsg
	call	DispMessage
	call	WriteAppSignature
Main_WriteFailed:
	ld	hl, writeFailedMsg
	jr	nz, Main_ShowFatalMessage
	ld	hl, finishedMsg
Main_ShowFatalMessage:
	BCALL	_FlashWriteDisable
	jp	ShowMessageWait
