;;; -*- TI-Asm -*-
;;;
;;; DuckSign - sign Flash applications on the TI-83 Plus
;;;
;;; Copyright (C) 2010 Benjamin Moody
;;;
;;; This program is free software: you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License as
;;; published by the Free Software Foundation; either version 3 of the
;;; License, or (at your option) any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;; General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program.  If not, see <http://www.gnu.org/licenses/>.

;; InitProgress:
;;
;; Show the initial progress bar on screen.
;;
;; Input:
;; - HL = total number of progress units (must be at least 80 for the
;;   display to work accurately)
;; - (penRow) = Y position to display progress bar (set to -1 to
;;   disable progress bar)
;;
;; Output:
;; - (progressCount) = time until next update
;; - (progressTotal) = total number of progress units
;; - (progressX) = LCD column
;; - (progressXBit) = current bit
;;
;; Destroys:
;; - AF, BC, DE, HL

InitProgress:
	ld	(progressTotal), hl
	dec	hl
	ld	(progressCount), hl
	ld	hl, 2001h
	ld	(progressXBit), hl

	ld	a, (penRow)
	add	a, 80h
	ret	c

	ld	d, a
	ld	e, h
	ld	bc, 0601h
	call	PutProgressPixels
	ld	l, 10
ShowProgressBar_Loop:
	ld	bc, 01FFh
	call	PutProgressPixels
	ld	bc, 0400h
	call	PutProgressPixels_Loop
	ld	bc, 01FFh
	call	PutProgressPixels_Loop
	dec	l
	jr	nz, ShowProgressBar_Loop
	ld	bc, 0680h
	jr	PutProgressPixels

;; UpdateProgress:
;;
;; Increment the progress bar counter.  If requested, draw a progress
;; bar on screen.
;;
;; Input:
;; - (progressCount) = time until next update
;; - (progressTotal) = total number of progress units
;; - (progressX) = LCD column
;; - (progressXBit) = current bit
;; - (penRow) = Y position to display progress bar (set to -1 to
;;   disable progress bar)
;;
;; Output:
;; - Carry flag clear
;;
;; Destroys:
;; - AF, BC, DE, HL

UpdateProgress:
	ld	hl, (progressCount)
	ld	de, -80
	add	hl, de
	ld	(progressCount),hl
	ccf
	ret	nc
	ld	de, (progressTotal)
	add	hl, de
	ld	(progressCount),hl

	ld	a, (penRow)
	sub	80h
	ret	nc
	inc	a
	ld	d, a

	ld	hl, progressXBit
	ld	a, (hl)
	rrca
	ld	(hl), a
	inc	hl
	jr	nc, UpdateProgress_NoInc
	inc	(hl)
UpdateProgress_NoInc:
	ld	e, (hl)
	dec	a
	cpl
	ld	c, a
	ld	b, 4
	;; fall through

;; PutProgressPixels:
;;
;; Write the given value to one or more consecutive bytes of LCD memory.
;;
;; Input:
;; - D = starting row
;; - E = starting column
;; - C = value to write
;; - B = number of copies
;;
;; Destroys:
;; - AF, B

PutProgressPixels:
	ld	a, d
	call	LCD_BUSY_QUICK
	out	(10h), a
	ld	a, e
	inc	de
	call	LCD_BUSY_QUICK
	out	(10h), a
	ld	a, (hl)
PutProgressPixels_Loop:
	ld	a, c
	call	LCD_BUSY_QUICK
	out	(11h), a
	djnz	PutProgressPixels_Loop
	or	a
	ret

