;;; -*- TI-Asm -*-
;;;
;;; DuckSign - sign Flash applications on the TI-83 Plus
;;;
;;; Copyright (C) 2010 Benjamin Moody
;;;
;;; This program is free software: you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License as
;;; published by the Free Software Foundation; either version 3 of the
;;; License, or (at your option) any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;; General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program.  If not, see <http://www.gnu.org/licenses/>.

  ifdef ENABLE_TIMER

AddAToCounter:			; (17 + 2)
	add	a, (hl)		; 7
	ld	(hl), a		; 7 + 1
	inc	hl		; 6
	ld	a, 0		; 7
	adc	a, (hl)		; 7
	ld	(hl), a		; 7 + 1
	inc	hl		; 6
	ld	a, 0		; 7
	adc	a, (hl)		; 7
	ld	(hl), a		; 7 + 1
	inc	hl		; 6
	ld	a, 0		; 7
	adc	a, (hl)		; 7
	ld	(hl), a		; 7 + 1
	ret			; 10
				; -- total = 122 + 6

Interrupt:			 ; (19 + 2)
	push	af		 ; 11 + 2
	 push	hl		 ; 11 + 2
	  in	a, (2)		 ; 11
	  add	a, a		 ; 4
	  jr	nc, Interrupt_BE ; 7
	  xor	a		 ; 4
	  out	(57h), a	 ; 11
	  out	(5Bh), a	 ; 11
	  in	a, (82h)	 ; 11
	  in	a, (84h)	 ; 11
	  in	a, (86h)	 ; 11
	  in	a, (4)		 ; 11
	  and	20h		 ; 7
				 ; -- total = 140 + 6

	  jr	z, Interrupt_NoTimer ; 7
	  in	a, (31h)	     ; 11
	  out	(31h), a	     ; 11
	  ld	a, 1		     ; 7
	  jr	Interrupt_TimerDone  ; 12
				     ; -- total = 48
	
Interrupt_NoTimer:		; (12)
	  ld	a, (hl)		; 7
	  ld	a, (hl)		; 7
	  ld	a, (hl)		; 7
	  ld	a, (hl)		; 7
	  nop			; 4
	  xor	a		; 4
				; -- total = 48
	
Interrupt_TimerDone:
	  ld	hl, timerExpiredCount ; 10
	  call	AddAToCounter	      ; 122 + 6
Interrupt_BE:
	  ld	a, 1		      ; 7
	  ld	hl, interruptCount    ; 10
	  call	AddAToCounter	      ; 122 + 6
	  ld	a, 8		      ; 7
	  out	(3), a		      ; 11
	  ld	a, 0Bh		      ; 7
	  out	(3), a		      ; 11
	  pop	hl		      ; 10
	 pop	af		      ; 10
	ei			      ; 4
	ret			      ; 10
				      ; -- total = 341 + 12

	;; ISR total: 529 + 18

StartTimer:
	di
	ld	a, 0C3h
	ld	(9898h), a
	ld	hl, Interrupt
	ld	(9899h), hl
	ld	a, 99h
	ld	i, a
	ld	bc, 256
	ld	h, a
	ld	l, c
	ld	(hl), 98h
	ld	d, a
	ld	e, b
	ldir
	im	2

	ld	hl, 0
	ld	(interruptCount), hl
	ld	(interruptCount + 2), hl
	ld	(timerExpiredCount), hl
	ld	(timerExpiredCount + 2), hl
	in	a, (2)
	add	a, a
	jr	nc, StartTimer_BE
	ld	a, 0A0h
	out	(30h), a
	ld	a, 3
	out	(31h), a
	ld	a, 255
	out	(32h), a ; timer expires every 64 * 255 = 16320 clock cycles
StartTimer_BE:
	ei
	ret

StopTimer:
	in	a, (2)
	add	a, a
	jr	nc, StopTimer_BE
	di
	in	a, (32h)
	push	af
	 ld	de, (timerExpiredCount)
	 ld	hl, (timerExpiredCount + 2)
	 push	hl
	  push	de
	   ld	de, (interruptCount)
	   ld	hl, (interruptCount + 2)
	   ei
	   call	SetXXXXXXXXOP2
	   BCALL _OP2ToOP1
	   in	a, (2Eh)
	   and	40h
	   ld	hl, 529
	   jr	z, StopTimer_NoMemDelay
	   in	a, (20h)
	   and	3
	   add	a, 29h
	   ld	c, a
	   in	a, (c)
	   and	2
	   jr	z, StopTimer_NoMemDelay
	   ld	hl, 529 + 18
StopTimer_NoMemDelay:
	   BCALL _SetXXXXOP2
	   BCALL _FPMult
	   BCALL _OP1ToOP6
	   pop	de
	  pop	hl
	 call	SetXXXXXXXXOP2
	 BCALL	_OP2ToOP1
	 ld	hl, 255
	 BCALL	_SetXXXXOP2
	 BCALL	_FPMult
	 pop	af
	cpl
	ld	l, a
	ld	h, 0
	BCALL	_SetXXXXOP2
	BCALL	_FPAdd
	ld	hl, 64
	BCALL	_SetXXXXOP2
	BCALL	_FPMult
	BCALL	_OP6ToOP2
	BCALL	_FPSub
	jr	StopTimer_CleanUp

StopTimer_BE:
	di
	ld	de, (interruptCount)
	ld	hl, (interruptCount + 2)
	ei
	call	SetXXXXXXXXOP2
	BCALL	_OP2ToOP1
StopTimer_CleanUp:
	di
	im	1
	in	a, (2)
	add	a, a
	ret	nc
	xor	a
	out	(32h), a
	out	(31h), a
	out	(30h), a
	ret

SetXXXXXXXXOP2:
	push	de
	 BCALL	_SetXXXXOP2
	 BCALL	_OP2ToOP1
	 ld	hl, 32768
	 BCALL	_SetXXXXOP2
	 BCALL	_FPMult
	 BCALL	_Times2
	 pop	hl
	BCALL	_SetXXXXOP2
	BCALL	_FPAdd
	ret

  endif
